'''

    Load CARS T+S and calculate Rossby radius of deformation

'''

import numpy as np
import scipy as sp
import scipy.io as io
from scipy import interpolate as interpolate
#from datetime import date
from Scientific.IO import NetCDF
from matplotlib import pyplot as plt
from mpl_toolkits import basemap as bm
import gsw

from ecoliver import nanmean

#
# Load T+S data
#

header = '/home/ecoliver/Desktop/data/cars/'
Tfile = 'temperature_cars2009a.nc'
Sfile = 'salinity_cars2009a.nc'

fileobj = NetCDF.NetCDFFile(header+Tfile, mode='r')

lon = fileobj.variables['lon'].getValue()
lat = fileobj.variables['lat'].getValue()
llon, llat = np.meshgrid(lon, lat)
depth = fileobj.variables['depth'].getValue()

T = fileobj.variables['mean'].getValue().astype(float)
T[T==fileobj.variables['mean']._FillValue.astype(float)] = np.nan
T = T*fileobj.variables['mean'].scale_factor + fileobj.variables['mean'].add_offset
fileobj.close()

fileobj = NetCDF.NetCDFFile(header+Sfile, mode='r')
S = fileobj.variables['mean'].getValue().astype(float)
S[S==fileobj.variables['mean']._FillValue.astype(float)] = np.nan
S = S*fileobj.variables['mean'].scale_factor + fileobj.variables['mean'].add_offset
fileobj.close()

#
# Domain
#

#i1 = 290
#i2 = 330
#j1 = 50
#j2 = 110

#lon = lon[i1:i2+1]
#lat = lat[j1:j2+1]
#T = T[:,j1:j2+1,i1:i2+1]
#S = S[:,j1:j2+1,i1:i2+1]

X = len(lon)
Y = len(lat)

#
# Calculate Rossby radius of deformation
#

# Depth and pressure
z = -depth
Z = len(z)
K = Z - np.sum(np.isnan(T), axis=0)
H = -np.append(np.nan, depth)[K]
p = np.nan*np.ones((Z,Y))
for j in range(Y):
    for k in range(Z):
        p[k,j] = gsw.p_from_z(z[k], lat[j])

# plt.pcolormesh(K, edgecolors='None', vmin=0, vmax=20)
# plt.pcolormesh(H, edgecolors='None', vmin=-5.5e3, vmax=0)
# plt.pcolormesh(p, edgecolors='None', vmin=0, vmax=5.5e3)

def f(lat):
    '''
    Coriolis parameter
    '''
    return 2*(2*np.pi/(60*60*24))*np.sin(lat*np.pi/180) # 1/s


# Calculate Rossby radius from T and S profiles
c1 = np.nan*np.zeros((Y,X))
cR = np.nan*np.zeros((Y,X))
N_neg_N2 = np.zeros((Y,X))
for j in range(Y):
    print j, Y
    for i in range(X):
        if K[j,i] > 1:
            # Calculate N2
            SA = gsw.SA_from_SP(S[0:K[j,i],j,i], p[0:K[j,i],j], lon[i], lat[j])
            CT = gsw.CT_from_t(SA, T[0:K[j,i],j,i], p[0:K[j,i],j])
            N2, p_N2 = gsw.Nsquared(SA, CT, p[0:K[j,i],j], lat[j])
            # Fix negative values of N2, following Chelton et al. (1998):
            # - Replace a negative value with the (pos) value at the next shallowest depth.
            # - Replace neg values at the shallowest depth with 10e-8
            N_neg_N2[j,i] = (N2<0).sum()
            if N2[0]<0:
                N2[0] = 1e-8
            for k in range(1,K[j,i]-1):
                if N2[k]<0:
                    N2[k] = N2[k-1]
            # Calculate c1, cR
            c1[j,i] = -np.trapz(np.sqrt(N2), gsw.z_from_p(p_N2, lat[j])) / np.pi # First baroclinic gravity wave speed [m/s]
            cR[j,i] = c1[j,i] / np.abs(f(lat[j])) # Rossby radius of deformation [m]

np.savez(header+'Rossby_radius', c1=c1, cR=cR, lon=lon, lat=lat, N_neg_N2=N_neg_N2, H=H, K=K)
io.savemat(header+'Rossby_radius.mat', mdict={'lon': lon, 'lat': lat, 'c1': c1, 'cR': cR, 'H': H, 'N_neg_N2': N_neg_N2, 'K': K})

plt.figure()
plt.pcolormesh(lon, lat, cR*1e-3, edgecolors='None', vmin=0, vmax=60)
plt.colorbar()

plt.figure()
plt.contourf(lon, lat, cR*1e-3, levels=np.arange(0,100,10))
plt.colorbar()

plt.figure()
plt.pcolormesh(lon, lat, N_neg_N2, edgecolors='None', vmin=0, vmax=N_neg_N2.max())
plt.colorbar()

plt.figure()
plt.pcolormesh(lon, lat, N_neg_N2/K, edgecolors='None', vmin=0, vmax=0.8)
plt.colorbar()

plt.figure()
plt.plot(H.flatten(), cR.flatten()*1e-3, 'ko')
plt.xlabel('Water depth [m]')
plt.ylabel('Rossby radius [km]')

plt.xlim(-1000,0)
plt.ylim(0,35)

plt.figure()
plt.plot(H.flatten(), (cR*np.abs(f(llat))).flatten()*1e-3, 'ko')
plt.xlabel('Water depth [m]')
plt.ylabel('Rossby radius * coriolis freq. [km/s]')

plt.xlim(-1000,0)
plt.ylim(0,35)

cR_depth = np.zeros(len(depth))
for k in range(len(depth)):
    cR_depth[k] = nanmean(cR.flatten()[H.flatten()==-depth[k]])

plt.figure()
plt.plot(depth, cR_depth*1e-3, 'ko')
plt.xlabel('Water depth [m]')
plt.ylabel('Rossby radius [km]')

cRf_depth = np.zeros(len(depth))
for k in range(len(depth)):
    cRf_depth[k] = nanmean((cR*np.abs(f(llat))).flatten()[H.flatten()==-depth[k]])

plt.figure()
plt.plot(depth, cRf_depth*1e-3, 'ko')
plt.xlabel('Water depth [m]')
plt.ylabel('Rossby radius * coriolis freq. [km/s]')

